@@ -46,6 +46,9 @@
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
+#ifdef HAVE_CRT_EXTERNS_H 
+#include <crt_externs.h> /* for _NSGetEnviron */
+#endif
 
 /* implement gutils's inline functions
  */
@@ -119,8 +122,8 @@ G_WIN32_DLLMAIN_FOR_DLL_NAME (static, dll_name)
 /**
  * glib_check_version:
  * @required_major: the required major version.
- * @required_minor: the required major version.
- * @required_micro: the required major version.
+ * @required_minor: the required minor version.
+ * @required_micro: the required micro version.
  *
  * Checks that the GLib library in use is compatible with the
  * given version. Generally you would pass in the constants
@@ -205,11 +208,44 @@ g_memmove (gpointer      dest,
 }
 #endif /* !HAVE_MEMMOVE && !HAVE_WORKING_BCOPY */
 
+#ifdef G_OS_WIN32
+#undef g_atexit
+#endif
+
 /**
  * g_atexit:
  * @func: the function to call on normal program termination.
  * 
  * Specifies a function to be called at normal program termination.
+ *
+ * Since GLib 2.8.2, on Windows g_atexit() actually is a preprocessor
+ * macro that maps to a call to the atexit() function in the C
+ * library. This means that in case the code that calls g_atexit(),
+ * i.e. atexit(), is in a DLL, the function will be called when the
+ * DLL is detached from the program. This typically makes more sense
+ * than that the function is called when the GLib DLL is detached,
+ * which happened earlier when g_atexit() was a function in the GLib
+ * DLL.
+ *
+ * The behaviour of atexit() in the context of dynamically loaded
+ * modules is not formally specified and varies wildly.
+ *
+ * On POSIX systems, calling g_atexit() (or atexit()) in a dynamically
+ * loaded module which is unloaded before the program terminates might
+ * well cause a crash at program exit.
+ *
+ * Some POSIX systems implement atexit() like Windows, and have each
+ * dynamically loaded module maintain an own atexit chain that is
+ * called when the module is unloaded.
+ *
+ * On other POSIX systems, before a dynamically loaded module is
+ * unloaded, the registered atexit functions (if any) residing in that
+ * module are called, regardless where the code that registered them
+ * resided. This is presumably the most robust approach.
+ *
+ * As can be seen from the above, for portability it's best to avoid
+ * calling g_atexit() (or atexit()) except in the main executable of a
+ * program.
  */
 void
 g_atexit (GVoidFunc func)
@@ -540,6 +576,11 @@ g_parse_debug_string  (const gchar     *string,
   guint result = 0;
   
   g_return_val_if_fail (string != NULL, 0);
+
+  /* this function is used by gmem.c/gslice.c initialization code,
+   * so introducing malloc dependencies here would require adaptions
+   * of those code portions.
+   */
   
   if (!g_ascii_strcasecmp (string, "all"))
     {
@@ -582,7 +623,7 @@ g_parse_debug_string  (const gchar     *string,
  * 
  * Return value: the name of the file without any leading directory components.
  *
- * Deprecated: Use g_path_get_basename() instead, but notice that
+ * Deprecated:2.2: Use g_path_get_basename() instead, but notice that
  * g_path_get_basename() allocates new memory for the returned string, unlike
  * this function which returns a pointer into the argument.
  **/
@@ -1119,6 +1160,30 @@ g_getenv (const gchar *variable)
 #endif /* G_OS_WIN32 */
 }
 
+/* _g_getenv_nomalloc
+ * this function does a getenv() without doing any kind of allocation
+ * through glib. it's suitable for chars <= 127 only (both, for the
+ * variable name and the contents) and for contents < 1024 chars in
+ * length. also, it aliases "" to a NULL return value.
+ **/
+const gchar*
+_g_getenv_nomalloc (const gchar *variable,
+                    gchar        buffer[1024])
+{
+  const gchar *retval = getenv (variable);
+  if (retval && retval[0])
+    {
+      gint l = strlen (retval);
+      if (l < 1024)
+        {
+          strncpy (buffer, retval, l);
+          buffer[l] = 0;
+          return buffer;
+        }
+    }
+  return NULL;
+}
+
 /**
  * g_setenv:
  * @variable: the environment variable to set, must not contain '='.
@@ -1229,15 +1294,16 @@ g_setenv (const gchar *variable,
 #endif /* G_OS_WIN32 */
 }
 
-#ifndef G_OS_WIN32
+#ifdef HAVE__NSGETENVIRON
+#define environ (*_NSGetEnviron())
+#elif !defined(G_OS_WIN32)
 
 /* According to the Single Unix Specification, environ is not in 
  * any system header, although unistd.h often declares it.
  */
 extern char **environ;
-
 #endif
-           
+
 /**
  * g_unsetenv:
  * @variable: the environment variable to remove, must not contain '='.
@@ -1340,6 +1406,7 @@ g_unsetenv (const gchar *variable)
 gchar **
 g_listenv (void)
 {
+#ifndef G_OS_WIN32
   gchar **result, *eq;
   gint len, i, j;
 
@@ -1357,6 +1424,73 @@ g_listenv (void)
   result[j] = NULL;
 
   return result;
+#else
+  gchar **result, *eq;
+  gint len = 0, i, j;
+
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *p, *q;
+
+      p = (wchar_t *) GetEnvironmentStringsW ();
+      if (p != NULL)
+	{
+	  q = p;
+	  while (*q)
+	    {
+	      q += wcslen (q) + 1;
+	      len++;
+	    }
+	}
+      result = g_new0 (gchar *, len + 1);
+
+      j = 0;
+      q = p;
+      while (*q)
+	{
+	  result[j] = g_utf16_to_utf8 (q, -1, NULL, NULL, NULL);
+	  if (result[j] != NULL)
+	    {
+	      eq = strchr (result[j], '=');
+	      if (eq && eq > result[j])
+		{
+		  *eq = '\0';
+		  j++;
+		}
+	      else
+		g_free (result[j]);
+	    }
+	  q += wcslen (q) + 1;
+	}
+      result[j] = NULL;
+      FreeEnvironmentStringsW (p);
+    }
+  else
+    {
+      len = g_strv_length (environ);
+      result = g_new0 (gchar *, len + 1);
+      
+      j = 0;
+      for (i = 0; i < len; i++)
+	{
+	  result[j] = g_locale_to_utf8 (environ[i], -1, NULL, NULL, NULL);
+	  if (result[j] != NULL)
+	    {
+	      eq = strchr (result[j], '=');
+	      if (eq && eq > result[j])
+		{
+		  *eq = '\0';
+		  j++;
+		}
+	      else
+		g_free (result[j]);
+	    }
+	}
+      result[j] = NULL;
+    }
+
+  return result;
+#endif
 }
 
 G_LOCK_DEFINE_STATIC (g_utils_global);
@@ -1441,259 +1575,285 @@ get_windows_directory_root (void)
 
 /* HOLDS: g_utils_global_lock */
 static void
-g_get_any_init (void)
+g_get_any_init_do (void)
 {
+  gchar hostname[100];
+
+  g_tmp_dir = g_strdup (g_getenv ("TMPDIR"));
+  if (!g_tmp_dir)
+    g_tmp_dir = g_strdup (g_getenv ("TMP"));
+  if (!g_tmp_dir)
+    g_tmp_dir = g_strdup (g_getenv ("TEMP"));
+
+#ifdef G_OS_WIN32
+  if (!g_tmp_dir)
+    g_tmp_dir = get_windows_directory_root ();
+#else  
+#ifdef P_tmpdir
   if (!g_tmp_dir)
     {
-      gchar hostname[100];
-
-      g_tmp_dir = g_strdup (g_getenv ("TMPDIR"));
-      if (!g_tmp_dir)
-	g_tmp_dir = g_strdup (g_getenv ("TMP"));
-      if (!g_tmp_dir)
-	g_tmp_dir = g_strdup (g_getenv ("TEMP"));
-      
-#ifdef G_OS_WIN32
-      if (!g_tmp_dir)
-	g_tmp_dir = get_windows_directory_root ();
-#else
-#ifdef P_tmpdir
-      if (!g_tmp_dir)
-	{
-	  gsize k;    
-	  g_tmp_dir = g_strdup (P_tmpdir);
-	  k = strlen (g_tmp_dir);
-	  if (k > 1 && G_IS_DIR_SEPARATOR (g_tmp_dir[k - 1]))
-	    g_tmp_dir[k - 1] = '\0';
-	}
+      gsize k;    
+      g_tmp_dir = g_strdup (P_tmpdir);
+      k = strlen (g_tmp_dir);
+      if (k > 1 && G_IS_DIR_SEPARATOR (g_tmp_dir[k - 1]))
+	g_tmp_dir[k - 1] = '\0';
+    }
 #endif
-      
-      if (!g_tmp_dir)
-	{
-	  g_tmp_dir = g_strdup ("/tmp");
-	}
+  
+  if (!g_tmp_dir)
+    {
+      g_tmp_dir = g_strdup ("/tmp");
+    }
 #endif	/* !G_OS_WIN32 */
-      
+  
 #ifdef G_OS_WIN32
-      /* We check $HOME first for Win32, though it is a last resort for Unix
-       * where we prefer the results of getpwuid().
-       */
-      g_home_dir = g_strdup (g_getenv ("HOME"));
+  /* We check $HOME first for Win32, though it is a last resort for Unix
+   * where we prefer the results of getpwuid().
+   */
+  g_home_dir = g_strdup (g_getenv ("HOME"));
 
-      /* Only believe HOME if it is an absolute path and exists */
-      if (g_home_dir)
+  /* Only believe HOME if it is an absolute path and exists */
+  if (g_home_dir)
+    {
+      if (!(g_path_is_absolute (g_home_dir) &&
+	    g_file_test (g_home_dir, G_FILE_TEST_IS_DIR)))
 	{
-	  if (!(g_path_is_absolute (g_home_dir) &&
-		g_file_test (g_home_dir, G_FILE_TEST_IS_DIR)))
-	    {
-	      g_free (g_home_dir);
-	      g_home_dir = NULL;
-	    }
-	}
-      
-      /* In case HOME is Unix-style (it happens), convert it to
-       * Windows style.
-       */
-      if (g_home_dir)
-	{
-	  gchar *p;
-	  while ((p = strchr (g_home_dir, '/')) != NULL)
-	    *p = '\\';
+	  g_free (g_home_dir);
+	  g_home_dir = NULL;
 	}
+    }
+  
+  /* In case HOME is Unix-style (it happens), convert it to
+   * Windows style.
+   */
+  if (g_home_dir)
+    {
+      gchar *p;
+      while ((p = strchr (g_home_dir, '/')) != NULL)
+	*p = '\\';
+    }
 
-      if (!g_home_dir)
-	{
-	  /* USERPROFILE is probably the closest equivalent to $HOME? */
-	  if (g_getenv ("USERPROFILE") != NULL)
-	    g_home_dir = g_strdup (g_getenv ("USERPROFILE"));
-	}
+  if (!g_home_dir)
+    {
+      /* USERPROFILE is probably the closest equivalent to $HOME? */
+      if (g_getenv ("USERPROFILE") != NULL)
+	g_home_dir = g_strdup (g_getenv ("USERPROFILE"));
+    }
 
-      if (!g_home_dir)
-	g_home_dir = get_special_folder (CSIDL_PROFILE);
-      
-      if (!g_home_dir)
-	g_home_dir = get_windows_directory_root ();
+  if (!g_home_dir)
+    g_home_dir = get_special_folder (CSIDL_PROFILE);
+  
+  if (!g_home_dir)
+    g_home_dir = get_windows_directory_root ();
 #endif /* G_OS_WIN32 */
-      
+  
 #ifdef HAVE_PWD_H
-      {
-	struct passwd *pw = NULL;
-	gpointer buffer = NULL;
-        gint error;
-	
+  {
+    struct passwd *pw = NULL;
+    gpointer buffer = NULL;
+    gint error;
+    gchar *logname;
+
 #  if defined (HAVE_POSIX_GETPWUID_R) || defined (HAVE_NONPOSIX_GETPWUID_R)
-        struct passwd pwd;
+    struct passwd pwd;
 #    ifdef _SC_GETPW_R_SIZE_MAX  
-	/* This reurns the maximum length */
-        glong bufsize = sysconf (_SC_GETPW_R_SIZE_MAX);
-	
-	if (bufsize < 0)
-	  bufsize = 64;
+    /* This reurns the maximum length */
+    glong bufsize = sysconf (_SC_GETPW_R_SIZE_MAX);
+    
+    if (bufsize < 0)
+      bufsize = 64;
 #    else /* _SC_GETPW_R_SIZE_MAX */
-        glong bufsize = 64;
+    glong bufsize = 64;
 #    endif /* _SC_GETPW_R_SIZE_MAX */
+
+    logname = (gchar *) g_getenv ("LOGNAME");
+        
+    do
+      {
+	g_free (buffer);
+	/* we allocate 6 extra bytes to work around a bug in 
+	 * Mac OS < 10.3. See #156446
+	 */
+	buffer = g_malloc (bufsize + 6);
+	errno = 0;
 	
-        do
-          {
-            g_free (buffer);
-	    /* we allocate 6 extra bytes to work around a bug in 
-	     * Mac OS < 10.3. See #156446
-             */
-	    buffer = g_malloc (bufsize + 6);
-	    errno = 0;
-	    
 #    ifdef HAVE_POSIX_GETPWUID_R
+	if (logname) {
+	  error = getpwnam_r (logname, &pwd, buffer, bufsize, &pw);
+	  if (!pw || (pw->pw_uid != getuid ())) {
+	    /* LOGNAME is lying, fall back to looking up the uid */
 	    error = getpwuid_r (getuid (), &pwd, buffer, bufsize, &pw);
-            error = error < 0 ? errno : error;
+	  }
+	} else {
+	  error = getpwuid_r (getuid (), &pwd, buffer, bufsize, &pw);
+	}
+	error = error < 0 ? errno : error;
 #    else /* HAVE_NONPOSIX_GETPWUID_R */
-       /* HPUX 11 falls into the HAVE_POSIX_GETPWUID_R case */
+   /* HPUX 11 falls into the HAVE_POSIX_GETPWUID_R case */
 #      if defined(_AIX) || defined(__hpux)
-	    error = getpwuid_r (getuid (), &pwd, buffer, bufsize);
-	    pw = error == 0 ? &pwd : NULL;
+	error = getpwuid_r (getuid (), &pwd, buffer, bufsize);
+	pw = error == 0 ? &pwd : NULL;
 #      else /* !_AIX */
-            pw = getpwuid_r (getuid (), &pwd, buffer, bufsize);
-            error = pw ? 0 : errno;
+	if (logname) {
+	  pw = getpwnam_r (logname, &pwd, buffer, bufsize);
+	  if (!pw || (pw->pw_uid != getuid ())) {
+	    /* LOGNAME is lying, fall back to looking up the uid */
+	    pw = getpwuid_r (getuid (), &pwd, buffer, bufsize);
+	  }
+	} else {
+	  pw = getpwuid_r (getuid (), &pwd, buffer, bufsize);
+	}
+	error = pw ? 0 : errno;
 #      endif /* !_AIX */            
 #    endif /* HAVE_NONPOSIX_GETPWUID_R */
-	    
-	    if (!pw)
-	      {
-		/* we bail out prematurely if the user id can't be found
-		 * (should be pretty rare case actually), or if the buffer
-		 * should be sufficiently big and lookups are still not
-		 * successfull.
-		 */
-		if (error == 0 || error == ENOENT)
-		  {
-		    g_warning ("getpwuid_r(): failed due to unknown user id (%lu)",
-			       (gulong) getuid ());
-		    break;
-		  }
-		if (bufsize > 32 * 1024)
-		  {
-		    g_warning ("getpwuid_r(): failed due to: %s.",
-			       g_strerror (error));
-		    break;
-		  }
-		
-		bufsize *= 2;
-	      }
-	  }
-	while (!pw);
-#  endif /* HAVE_POSIX_GETPWUID_R || HAVE_NONPOSIX_GETPWUID_R */
 	
 	if (!pw)
 	  {
-	    setpwent ();
-	    pw = getpwuid (getuid ());
-	    endpwent ();
-	  }
-	if (pw)
-	  {
- 	    g_user_name = g_strdup (pw->pw_name);
-
-	    if (pw->pw_gecos && *pw->pw_gecos != '\0') 
+	    /* we bail out prematurely if the user id can't be found
+	     * (should be pretty rare case actually), or if the buffer
+	     * should be sufficiently big and lookups are still not
+	     * successfull.
+	     */
+	    if (error == 0 || error == ENOENT)
 	      {
-		gchar **gecos_fields;
-		gchar **name_parts;
-
-		/* split the gecos field and substitute '&' */
-		gecos_fields = g_strsplit (pw->pw_gecos, ",", 0);
-		name_parts = g_strsplit (gecos_fields[0], "&", 0);
-		pw->pw_name[0] = g_ascii_toupper (pw->pw_name[0]);
-		g_real_name = g_strjoinv (pw->pw_name, name_parts);
-		g_strfreev (gecos_fields);
-		g_strfreev (name_parts);
+		g_warning ("getpwuid_r(): failed due to unknown user id (%lu)",
+			   (gulong) getuid ());
+		break;
 	      }
-
-	    if (!g_home_dir)
-	      g_home_dir = g_strdup (pw->pw_dir);
+	    if (bufsize > 32 * 1024)
+	      {
+		g_warning ("getpwuid_r(): failed due to: %s.",
+			   g_strerror (error));
+		break;
+	      }
+	    
+	    bufsize *= 2;
 	  }
-	g_free (buffer);
       }
-      
+    while (!pw);
+#  endif /* HAVE_POSIX_GETPWUID_R || HAVE_NONPOSIX_GETPWUID_R */
+    
+    if (!pw)
+      {
+	setpwent ();
+	pw = getpwuid (getuid ());
+	endpwent ();
+      }
+    if (pw)
+      {
+	g_user_name = g_strdup (pw->pw_name);
+
+	if (pw->pw_gecos && *pw->pw_gecos != '\0') 
+	  {
+	    gchar **gecos_fields;
+	    gchar **name_parts;
+
+	    /* split the gecos field and substitute '&' */
+	    gecos_fields = g_strsplit (pw->pw_gecos, ",", 0);
+	    name_parts = g_strsplit (gecos_fields[0], "&", 0);
+	    pw->pw_name[0] = g_ascii_toupper (pw->pw_name[0]);
+	    g_real_name = g_strjoinv (pw->pw_name, name_parts);
+	    g_strfreev (gecos_fields);
+	    g_strfreev (name_parts);
+	  }
+
+	if (!g_home_dir)
+	  g_home_dir = g_strdup (pw->pw_dir);
+      }
+    g_free (buffer);
+  }
+  
 #else /* !HAVE_PWD_H */
-      
+  
 #ifdef G_OS_WIN32
-      if (G_WIN32_HAVE_WIDECHAR_API ())
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      guint len = UNLEN+1;
+      wchar_t buffer[UNLEN+1];
+      
+      if (GetUserNameW (buffer, (LPDWORD) &len))
 	{
-	  guint len = UNLEN+1;
-	  wchar_t buffer[UNLEN+1];
-	  
-	  if (GetUserNameW (buffer, (LPDWORD) &len))
-	    {
-	      g_user_name = g_utf16_to_utf8 (buffer, -1, NULL, NULL, NULL);
-	      g_real_name = g_strdup (g_user_name);
-	    }
+	  g_user_name = g_utf16_to_utf8 (buffer, -1, NULL, NULL, NULL);
+	  g_real_name = g_strdup (g_user_name);
 	}
-      else
+    }
+  else
+    {
+      guint len = UNLEN+1;
+      char buffer[UNLEN+1];
+      
+      if (GetUserNameA (buffer, (LPDWORD) &len))
 	{
-	  guint len = UNLEN+1;
-	  char buffer[UNLEN+1];
-	  
-	  if (GetUserNameA (buffer, (LPDWORD) &len))
-	    {
-	      g_user_name = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
-	      g_real_name = g_strdup (g_user_name);
-	    }
+	  g_user_name = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+	  g_real_name = g_strdup (g_user_name);
 	}
+    }
 #endif /* G_OS_WIN32 */
 
 #endif /* !HAVE_PWD_H */
 
 #ifndef G_OS_WIN32
-      if (!g_home_dir)
-	g_home_dir = g_strdup (g_getenv ("HOME"));
+  if (!g_home_dir)
+    g_home_dir = g_strdup (g_getenv ("HOME"));
 #endif
 
 #ifdef __EMX__
-      /* change '\\' in %HOME% to '/' */
-      g_strdelimit (g_home_dir, "\\",'/');
+  /* change '\\' in %HOME% to '/' */
+  g_strdelimit (g_home_dir, "\\",'/');
 #endif
-      if (!g_user_name)
-	g_user_name = g_strdup ("somebody");
-      if (!g_real_name)
-	g_real_name = g_strdup ("Unknown");
+  if (!g_user_name)
+    g_user_name = g_strdup ("somebody");
+  if (!g_real_name)
+    g_real_name = g_strdup ("Unknown");
 
+  {
 #ifndef G_OS_WIN32
-      if (gethostname (hostname, sizeof (hostname)) == -1)
-	g_host_name = g_strdup ("unknown");
-      else
-	g_host_name = g_strdup (hostname);
+    gboolean hostname_fail = (gethostname (hostname, sizeof (hostname)) == -1);
 #else
-      {
-	DWORD size = sizeof (hostname);
-	
-	if (!GetComputerName (hostname, &size))
-	  g_host_name = g_strdup ("unknown");
-	else
-	  g_host_name = g_strdup (hostname);
-      }
+    DWORD size = sizeof (hostname);
+    gboolean hostname_fail = (!GetComputerName (hostname, &size));
 #endif
+    g_host_name = g_strdup (hostname_fail ? "localhost" : hostname);
+  }
 
 #ifdef G_OS_WIN32
-      g_tmp_dir_cp = g_locale_from_utf8 (g_tmp_dir, -1, NULL, NULL, NULL);
-      g_user_name_cp = g_locale_from_utf8 (g_user_name, -1, NULL, NULL, NULL);
-      g_real_name_cp = g_locale_from_utf8 (g_real_name, -1, NULL, NULL, NULL);
+  g_tmp_dir_cp = g_locale_from_utf8 (g_tmp_dir, -1, NULL, NULL, NULL);
+  g_user_name_cp = g_locale_from_utf8 (g_user_name, -1, NULL, NULL, NULL);
+  g_real_name_cp = g_locale_from_utf8 (g_real_name, -1, NULL, NULL, NULL);
 
-      if (!g_tmp_dir_cp)
-	g_tmp_dir_cp = g_strdup ("\\");
-      if (!g_user_name_cp)
-	g_user_name_cp = g_strdup ("somebody");
-      if (!g_real_name_cp)
-	g_real_name_cp = g_strdup ("Unknown");
+  if (!g_tmp_dir_cp)
+    g_tmp_dir_cp = g_strdup ("\\");
+  if (!g_user_name_cp)
+    g_user_name_cp = g_strdup ("somebody");
+  if (!g_real_name_cp)
+    g_real_name_cp = g_strdup ("Unknown");
 
-      /* home_dir might be NULL, unlike tmp_dir, user_name and
-       * real_name.
-       */
-      if (g_home_dir)
-	g_home_dir_cp = g_locale_from_utf8 (g_home_dir, -1, NULL, NULL, NULL);
-      else
-	g_home_dir_cp = NULL;
+  /* home_dir might be NULL, unlike tmp_dir, user_name and
+   * real_name.
+   */
+  if (g_home_dir)
+    g_home_dir_cp = g_locale_from_utf8 (g_home_dir, -1, NULL, NULL, NULL);
+  else
+    g_home_dir_cp = NULL;
 #endif /* G_OS_WIN32 */
-    }
 }
 
+static inline void
+g_get_any_init (void)
+{
+  if (!g_tmp_dir)
+    g_get_any_init_do ();
+}
+
+static inline void
+g_get_any_init_locked (void)
+{
+  G_LOCK (g_utils_global);
+  g_get_any_init ();
+  G_UNLOCK (g_utils_global);
+}
+
+
 /**
  * g_get_user_name:
  *
@@ -1707,11 +1867,7 @@ g_get_any_init (void)
 G_CONST_RETURN gchar*
 g_get_user_name (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
-  
+  g_get_any_init_locked ();
   return g_user_name;
 }
 
@@ -1729,11 +1885,7 @@ g_get_user_name (void)
 G_CONST_RETURN gchar*
 g_get_real_name (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
- 
+  g_get_any_init_locked ();
   return g_real_name;
 }
 
@@ -1751,11 +1903,7 @@ g_get_real_name (void)
 G_CONST_RETURN gchar*
 g_get_home_dir (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
-  
+  g_get_any_init_locked ();
   return g_home_dir;
 }
 
@@ -1774,11 +1922,7 @@ g_get_home_dir (void)
 G_CONST_RETURN gchar*
 g_get_tmp_dir (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
-  
+  g_get_any_init_locked ();
   return g_tmp_dir;
 }
 
@@ -1795,7 +1939,7 @@ g_get_tmp_dir (void)
  * of the machine is changed while an application is running, the
  * return value from this function does not change. The returned
  * string is owned by GLib and should not be modified or freed. If no
- * name can be determined, a default fixed string "unknown" is
+ * name can be determined, a default fixed string "localhost" is
  * returned.
  *
  * Returns: the host name of the machine.
@@ -1805,11 +1949,7 @@ g_get_tmp_dir (void)
 const gchar *
 g_get_host_name (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
-
+  g_get_any_init_locked ();
   return g_host_name;
 }
 
@@ -1988,14 +2128,13 @@ g_get_user_data_dir (void)
 #endif
       if (!data_dir || !data_dir[0])
 	{
-	  if (!g_tmp_dir)
-	    g_get_any_init ();
+	  g_get_any_init ();
 
 	  if (g_home_dir)
 	    data_dir = g_build_filename (g_home_dir, ".local", 
 					 "share", NULL);
 	  else
-	    data_dir = g_build_filename (g_tmp_dir, g_user_name, ".local",
+	    data_dir = g_build_filename (g_tmp_dir, g_user_name, ".local", 
 					 "share", NULL);
 	}
 
@@ -2042,9 +2181,8 @@ g_get_user_config_dir (void)
 #endif
       if (!config_dir || !config_dir[0])
 	{
-	  if (!g_tmp_dir)
-	    g_get_any_init ();
-	  
+	  g_get_any_init ();
+
 	  if (g_home_dir)
 	    config_dir = g_build_filename (g_home_dir, ".config", NULL);
 	  else
@@ -2093,9 +2231,8 @@ g_get_user_cache_dir (void)
 #endif
       if (!cache_dir || !cache_dir[0])
 	{
-	  if (!g_tmp_dir)
-	    g_get_any_init ();
-
+	  g_get_any_init ();
+	
 	  if (g_home_dir)
 	    cache_dir = g_build_filename (g_home_dir, ".cache", NULL);
 	  else
@@ -2560,10 +2697,10 @@ _g_compute_locale_variants (const gchar *locale)
 {
   GSList *retval = NULL;
 
-  gchar *language;
-  gchar *territory;
-  gchar *codeset;
-  gchar *modifier;
+  gchar *language = NULL;
+  gchar *territory = NULL;
+  gchar *codeset = NULL;
+  gchar *modifier = NULL;
 
   guint mask;
   guint i;
@@ -2991,11 +3128,7 @@ g_unsetenv (const gchar *variable)
 G_CONST_RETURN gchar*
 g_get_user_name (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
-  
+  g_get_any_init_locked ();
   return g_user_name_cp;
 }
 
@@ -3004,11 +3137,7 @@ g_get_user_name (void)
 G_CONST_RETURN gchar*
 g_get_real_name (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
- 
+  g_get_any_init_locked ();
   return g_real_name_cp;
 }
 
@@ -3017,11 +3146,7 @@ g_get_real_name (void)
 G_CONST_RETURN gchar*
 g_get_home_dir (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
-
+  g_get_any_init_locked ();
   return g_home_dir_cp;
 }
 
@@ -3030,11 +3155,7 @@ g_get_home_dir (void)
 G_CONST_RETURN gchar*
 g_get_tmp_dir (void)
 {
-  G_LOCK (g_utils_global);
-  if (!g_tmp_dir)
-    g_get_any_init ();
-  G_UNLOCK (g_utils_global);
-
+  g_get_any_init_locked ();
   return g_tmp_dir_cp;
 }
 
